import 'package:draft_flutter/draft/draft_objects.dart';

/// Use this object to render text if the text has inline style
class TextBlock {
  final String text;
  final String style;
  final String type;

  TextBlock({this.text, this.style, this.type});
}

extension ContentBlockExtension on RawDraftContentBlock {
  List<RawDraftEntityKeyStringAny> getEntities(
      Map<String, RawDraftEntityKeyStringAny> entityMap) {
    if (this.entityRanges.length == 0) {
      throw UnsupportedError("Content block is not supported");
    }

    return this
        .entityRanges
        .map((e) => entityMap[e.key.toInt().toString()])
        .toList();
  }

  /// Get text blocks by inline styles
  List<TextBlock> get textBlocks {
    List<TextBlock> blocks = [];
    var tempText = "${this.text}";
    for (var style in this.inlineStyleRanges) {
      var subText = tempText.substring(
          style.offset.toInt(), (style.offset + style.length).toInt());
      blocks.add(TextBlock(text: subText, style: style.style, type: this.type));
      tempText = tempText.replaceRange(
          style.offset.toInt(), (style.offset + style.length).toInt(), "");
    }

    return [TextBlock(text: tempText, type: this.type), ...blocks];
  }
}
