import 'package:draft_flutter/draft/draft_objects.dart';
import 'package:draft_flutter/draft/extensions.dart';
import 'package:draft_flutter/views/AtomicView.dart';
import 'package:draft_flutter/views/blockquote.dart';
import 'package:flutter/material.dart';

class DraftView extends StatefulWidget {
  /// Raw Draft JS Content State object
  final Map<String, dynamic> rawContentState;

  DraftView({@required this.rawContentState});

  @override
  _DraftViewState createState() => _DraftViewState();
}

class _DraftViewState extends State<DraftView> {
  DraftObject object;
  int currentNum = 0;

  @override
  void initState() {
    super.initState();
    object = DraftObject.fromJson(widget.rawContentState);
  }

  /// render text style
  TextStyle _renderStyle(String style, {String type = "unstyled"}) {
    var normalStyle = Theme.of(context).primaryTextTheme.bodyText1;
    if (type == "header-one") {
      normalStyle = Theme.of(context).primaryTextTheme.headline3;
    } else if (type == "header-two") {
      normalStyle = Theme.of(context).primaryTextTheme.headline4;
    } else if (type == "header-three") {
      normalStyle = Theme.of(context).primaryTextTheme.headline5;
    }

    if (style == "BOLD") {
      return normalStyle.copyWith(
          fontWeight: FontWeight.bold, fontSize: normalStyle.fontSize + 4);
    }

    return normalStyle;
  }

  InlineSpan _renderText(RawDraftContentBlock block,
      {bool withNewline = true}) {
    if (block.text.isEmpty) {
      return TextSpan(text: "\n");
    }
    List<TextSpan> spans = [];

    for (var b in block.textBlocks) {
      spans.add(
        TextSpan(
          text: b.text,
          style: _renderStyle(b.style, type: b.type),
        ),
      );
    }
    if (withNewline) {
      spans.add(TextSpan(text: "\n"));
    }

    return TextSpan(
      children: spans,
    );
  }

  InlineSpan _renderSpan(RawDraftContentBlock block,
      Map<String, RawDraftEntityKeyStringAny> entityMap) {
    switch (block.type) {
      case "ordered-list-item":
        currentNum += 1;
        return TextSpan(
          children: [
            _renderText(block.copyWith(text: "  $currentNum. "),
                withNewline: false),
            _renderText(block)
          ],
        );
      case "unordered-list-item":
        currentNum = 0;
        return TextSpan(
          children: [
            _renderText(block.copyWith(text: "    ● "), withNewline: false),
            _renderText(block)
          ],
        );
      case "blockquote":
        currentNum = 0;
        return WidgetSpan(
          child: BlockQuote(
            inlineSpan: _renderText(block, withNewline: false),
          ),
        );
      case "atomic":
        currentNum = 0;
        return WidgetSpan(
          child: AtomicView(
            block: block,
            entityMap: entityMap,
          ),
        );
      default:
        return _renderText(block);
    }
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      child: RichText(
        text: TextSpan(
          children: object.blocks
              .map(
                (e) => _renderSpan(e, object.entityMap),
              )
              .toList(),
        ),
      ),
    );
  }
}
