import 'package:flutter_test/flutter_test.dart';
import 'package:draft_flutter/draft_flutter.dart';

import 'test_data.dart';

void main() {
  group("Test Convert from json to draft object", () {
    DraftObject object;

    test("Test parsing", () {
      object = DraftObject.fromJson(testData);
      expect(object.blocks.length, 16);
      expect(object.entityMap.length, 2);
    });

    test("Get entity", () {
      object = DraftObject.fromJson(testData);
      var block = object.blocks[14];
      var entities = block.getEntities(object.entityMap);
      expect(entities.length, 1);
      expect(entities[0].type, 'image');
      expect(entities[0].data['src'],
          'https://sirileepage-website-data.s3.amazonaws.com/static/image/2020/02/10/33319999c2b142f8b7d20eb9f6ac4de0.jpeg');
    });

    test("Get entity on unsupport block", () {
      object = DraftObject.fromJson(testData);
      var block = object.blocks[0];
      expect(() => block.getEntities(object.entityMap), throwsUnsupportedError);
    });

    test("inline styles", () {
      object = DraftObject.fromJson(testData);
      var block = object.blocks[8];
      List<TextBlock> blocks = block.textBlocks;
      expect(blocks.length, 2);
      expect(blocks[0].text, "当然还有");
      expect(blocks[1].text, "加粗");
      expect(blocks[0].type, "unstyled");
    });

    test("inline styles", () {
      object = DraftObject.fromJson(testData);
      var block = object.blocks[10];
      List<TextBlock> blocks = block.textBlocks;
      expect(blocks.length, 1);
      expect(blocks[0].type, "header-one");
    });
  });
}
